package de.mattv.fileserver;

import de.mattv.fileserver.util.NonNull;
import jakarta.annotation.Nullable;

public class Response<T> {
    public static <S> Response<S> e(@NonNull String v) { return new Response<>(v, null); }
    public static <S> Response<S> o(@NonNull S v) { return new Response<>(null, v); }

    public final @Nullable String e;
    public final @Nullable T o;

    private Response(@Nullable String e, @Nullable T o) {
        this.e = e;
        this.o = o;
    }

    public record Login(
            @NonNull boolean otpNeeded,
            @Nullable String token
    ) {}

    public record Session(
            @NonNull String name,
            @NonNull boolean tfaEnabled,
            @NonNull boolean admin,
            @NonNull boolean sudo
    ) {}

    public record UserInfo(
            @NonNull long id,
            @NonNull String name,
            @NonNull boolean tfaEnabled,
            @NonNull boolean enabled,
            @NonNull boolean admin
    ) {}

    public record CreateNodeInfo(
            @NonNull long id,
            @NonNull boolean exists,
            @NonNull boolean isFile
    ) {}

    public record PathSegment(
            @NonNull String name,
            @Nullable Long id
    ) {}

    public record Node(
            @NonNull long id,
            @NonNull String name,
            @NonNull boolean file,
            @NonNull boolean preview,
            @Nullable Long size,
            @Nullable Long parent,
            @Nullable Node[] children
    ) {
        public static Node from(de.mattv.fileserver.data.Node node, Node[] children) {
            return new Node(
                    node.id,
                    node.name,
                    node.isFile,
                    node.hasPreview,
                    node.size,
                    node.parent == null ? null : node.parent.id,
                    children
            );
        }
    }
}
