package de.mattv.fileserver.data.converter;

import com.thoughtworks.xstream.io.HierarchicalStreamReader;
import com.thoughtworks.xstream.io.HierarchicalStreamWriter;
import de.mattv.fileserver.data.Data;
import de.mattv.fileserver.data.User;
import de.mattv.fileserver.util.AutoCloseLock;
import de.mattv.fileserver.util.NonNull;
import lombok.extern.slf4j.Slf4j;

@Slf4j
public class DataConverter {
    private static final String NODE_NAME = "data";
    private static final String ATTR_VERSION_NAME = "version";
    private static final String ATTR_NEXT_USER_ID_NAME = "next-user";

    public static void toXml(@NonNull HierarchicalStreamWriter writer) {
        try (AutoCloseLock ignored = new AutoCloseLock(Data.USER_LOCK.readLock())) {
            writer.startNode(NODE_NAME);
            writer.addAttribute(ATTR_VERSION_NAME, String.valueOf(Data.CURRENT_VERSION));
            writer.addAttribute(ATTR_NEXT_USER_ID_NAME, String.valueOf(Data.nextUserId));
            Data.USERS.values().forEach(user -> UserConverter.toXml(user, writer));
            writer.endNode();
        }
    }

    public static void fromXML(@NonNull HierarchicalStreamReader reader) {
        ConverterUtils.hasError = false;

        if (!reader.getNodeName().equals(NODE_NAME)) {
            log.error("Trying to parse user from xml type {}", reader.getNodeName());
            ConverterUtils.dumpConversionStackFrames();
        }

        ConverterUtils.push("Data");

        long version = ConverterUtils.getLong(reader, ATTR_VERSION_NAME);
        if (version != Data.CURRENT_VERSION) {
            log.error("Data version is {}, current version is {}.", version, Data.CURRENT_VERSION);
            ConverterUtils.hasError = true;
            return;
        }
        Data.nextUserId = ConverterUtils.getLong(reader, ATTR_NEXT_USER_ID_NAME);

        while (reader.hasMoreChildren()) {
            User child = UserConverter.fromXml(reader);
            Data.USERS.put(child.id, child);
        }

        ConverterUtils.pop();
    }
}
