package de.mattv.fileserver.routes.auth;

import de.mattv.fileserver.Response;
import de.mattv.fileserver.data.Data;
import de.mattv.fileserver.data.User;
import de.mattv.fileserver.services.TFAMailer;
import de.mattv.fileserver.security.TokenService;
import de.mattv.fileserver.util.PublicRestController;
import de.mattv.fileserver.util.AutoCloseLock;
import jakarta.annotation.Nullable;
import de.mattv.fileserver.util.NonNull;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

@PublicRestController
public class Login {
    private final TFAMailer mailer;

    public Login(TFAMailer mailer) {
        this.mailer = mailer;
    }

    private record Body(@NonNull String username, @NonNull String password, @Nullable String otp) {}

    @PostMapping("/auth/login")
    private @NonNull Response<Response.Login> login(@RequestBody Body body) {
        try (AutoCloseLock ignored = new AutoCloseLock(Data.USER_LOCK.readLock())) {
            User user = Data.findUser(body.username);
            if (user == null)
                return Response.e("Invalid username or password");

            if (!AuthUtils.PW_ENCODER.matches(body.password, user.password))
                return Response.e("Invalid username or password");

            if (AuthUtils.PW_ENCODER.upgradeEncoding(user.password)) {
                user.password = AuthUtils.PW_ENCODER.encode(body.password);
                Data.save();
            }

            if (!user.enabled)
                return Response.e("User is disabled");

            if (user.tfaSecret != null) {
                if (body.otp == null) {
                    if (user.tfaSecret.isEmpty())
                        mailer.sendMail(user);
                    return Response.o(new Response.Login(true, null));
                } else {
                    boolean ok = user.tfaSecret.isEmpty()
                            ? mailer.checkCode(user.id, body.otp)
                            : AuthUtils.TOTP_VERIFIER.isValidCode(user.tfaSecret, body.otp);
                    if (!ok)
                        return Response.e("Invalid code");
                }
            }

            return Response.o(new Response.Login(false, TokenService.createToken(user)));
        }
    }
}
