package de.mattv.fileserver.routes.auth;

import de.mattv.fileserver.data.Data;
import de.mattv.fileserver.data.User;
import de.mattv.fileserver.services.RecoveryMailer;
import de.mattv.fileserver.security.TokenService;
import de.mattv.fileserver.util.PublicRestController;
import de.mattv.fileserver.util.AutoCloseLock;
import de.mattv.fileserver.util.NonNull;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.Optional;

@PublicRestController
public class Recovery {
    private final RecoveryMailer mailer;

    public Recovery(RecoveryMailer mailer) {
        this.mailer = mailer;
    }

    @PostMapping("/auth/send_recovery_key")
    private void sendKey(@RequestBody String username) {
        try (AutoCloseLock ignored = new AutoCloseLock(Data.USER_LOCK.readLock())) {
            User user = Data.findUser(username);
            if (user != null)
                mailer.sendMail(user);
        }
    }

    private record Body(@NonNull String key, @NonNull String password) {}

    @PostMapping("/auth/reset_password")
    private @NonNull Optional<String> recover(@RequestBody Body body) {
        try (AutoCloseLock ignored = new AutoCloseLock(Data.USER_LOCK.readLock())) {
            User user = mailer.checkCode(body.key);
            if (user == null)
                return Optional.of("Invalid key");
            if (body.password.length() < 6)
                return Optional.of("Password must be at least 6 characters");

            user.password = AuthUtils.PW_ENCODER.encode(body.password);
            TokenService.logoutAll(user.id);
            Data.save();
            return Optional.empty();
        }
    }
}
