package de.mattv.fileserver.routes.fs;

import de.mattv.fileserver.data.Node;
import de.mattv.fileserver.data.User;
import de.mattv.fileserver.util.AuthUser;
import de.mattv.fileserver.util.AutoCloseLock;
import de.mattv.fileserver.util.UserRestController;
import de.mattv.fileserver.util.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Base64;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.io.IOException;
import java.nio.file.Files;
import java.util.Optional;

@Slf4j
@UserRestController
public class Preview {
    @PostMapping("/fs/preview")
    private @NonNull Optional<String> preview(@AuthUser User user, @RequestBody long id) {
        try (AutoCloseLock ignored = new AutoCloseLock(user.nodeLock.readLock())) {
            Node node = user.nodes.get(id);
            if (node == null || !node.hasPreview) return Optional.empty();
            byte[] content = Files.readAllBytes(node.previewFile.toPath());
            return Optional.of(Base64.encodeBase64String(content));
        } catch (IOException e) {
            log.warn("Failed to read preview file", e);
            return Optional.empty();
        }
    }
}
