package de.mattv.fileserver.security;

import de.mattv.fileserver.data.Token;
import de.mattv.fileserver.data.User;
import de.mattv.fileserver.util.NonNull;
import org.apache.commons.lang3.RandomStringUtils;
import org.springframework.lang.Nullable;
import org.springframework.scheduling.annotation.Scheduled;

import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;

public class TokenService {
    private static final ConcurrentHashMap<String, Token> TOKENS = new ConcurrentHashMap<>();

    @Scheduled(fixedRate = 2, timeUnit = TimeUnit.HOURS)
    private static void cleanExpiredTokens() {
        TOKENS.entrySet().removeIf(entry -> entry.getValue().expired());
    }

    public static String createToken(@NonNull User user) {
        String token = RandomStringUtils.random(32, true, true);
        TOKENS.put(token, new Token(token, user));
        return token;
    }

    public static @Nullable Token getToken(@NonNull String token) {
        Token found = TOKENS.get(token);
        if (found != null && found.expired()) {
            TOKENS.remove(token);
            found = null;
        } else if (found != null) {
            found.refresh();
        }
        return found;
    }

    public static void logout(@NonNull Token token) {
        TOKENS.remove(token.getToken());
    }

    public static void logoutAll(long id) {
        TOKENS.entrySet().removeIf(entry -> {
            Token token = entry.getValue();
            return token.getUser().id == id || token.getRealUser().id == id;
        });
    }
}
